//
//  rn_Texture1.fsh
//  Rayne
//
//  Copyright 2013 by Überpixel. All rights reserved.
//  Unauthorized use is punishable by torture, mutilation, and vivisection.
//

#version 150
precision highp float;

#include <shader/rn_Lighting.fsh>

uniform sampler2D mTexture0;
uniform sampler2D mTexture1;
uniform sampler2D mTexture2;

#if defined(RN_SPECULARITY)
	uniform vec4 specular;
#endif

in vec2 vertTexcoord;
in vec3 vertNormal;
in vec3 vertPosition;

out vec4 fragColor0;

void main()
{
	vec3 walltexcoord = vertPosition.xyz*0.2f;
	
	vec4 colorX = texture(mTexture1, walltexcoord.yz);
	vec2 normalX = texture(mTexture2, walltexcoord.yz).xy-0.5;
	vec4 colorY = texture(mTexture1, walltexcoord.zx);
	vec2 normalY = texture(mTexture2, walltexcoord.zx).xy-0.5;
	vec4 colorZ = texture(mTexture1, walltexcoord.xy);
	vec2 normalZ = texture(mTexture2, walltexcoord.xy).xy-0.5;
	vec4 colorYbottom = texture(mTexture0, vertPosition.zx*0.4f);
	
	vec3 normal = normalize(vertNormal);
	vec3 projnormal = normal*1.5;
	projnormal *= projnormal*projnormal*projnormal;
	projnormal = clamp(projnormal, 0.0, 1.0);
	float bottomBlend = 1.0-clamp(sign(normal.y)*projnormal.y, 0.0, 1.0);
	projnormal /= projnormal.x+projnormal.y+projnormal.z;
	
	vec4 color0 = colorX*projnormal.x+colorZ*projnormal.z+colorY*projnormal.y;
	color0 = mix(colorYbottom, color0, bottomBlend);
	
	vec3 bumpX = vec3(0, normalX.x, normalX.y);
	vec3 bumpY = vec3(normalY.y, 0, normalY.x);
	vec3 bumpZ = vec3(normalZ.x, normalZ.y, 0);
	vec3 normal0 = bumpX*projnormal.x+bumpZ*projnormal.z+bumpY*projnormal.y;
	normal0 = normalize(normal+normal0*1.5f*bottomBlend);

	#if defined(RN_LIGHTING)
		#if defined(RN_SPECULARITY)
			vec4 spec;
			spec.rgb = specular.rgb*bottomBlend;
			spec.a = specular.a;
		#else
			vec4 spec = vec4(0.0);
		#endif
		rn_Lighting(color0, spec, normal0, vertPosition);
	#endif
	
	color0.r += clamp(-(vertPosition.y+13.0)/200.0, 0.0f, 1.0);
	
	float dist = length(vertPosition-vec3(250.8, 55.5, -142.7));
	dist = 1.0-clamp(dist/10.0, 0.0, 1.0);
	dist *= dist*5.0f;
	color0.rgb += vec3(dist);
	
	fragColor0 = color0;
}
