//
//  RASource.h
//  rayne-openal
//
//  Created by Sidney Just on 07.08.13.
//  Copyright (c) 2013 Sidney Just. All rights reserved.
//

#ifndef __rayne_openal__RASource__
#define __rayne_openal__RASource__

#include <Rayne.h>
#include "RASound.h"

#define kRNOpenALBufferCount 8
#define kRNSourceStoppedPlaying RNCSTR("kRNSourceStoppedPlaying")

namespace RN
{
	namespace openal
	{
		class Source : public SceneNode
		{
		public:
			enum
			{
				Looping  = (1 << 0),
				Relative = (1 << 1)
			};
			typedef uint32 Flags;
			
			Source(Sound *sound);
			~Source() override;
			
			void Play();
			void Stop();
			void Pause();
			
			bool IsPlaying();
			bool IsPaused();
			
			void SetFlags(Flags flags);
			void SetGain(float gain);
			void SetPitch(float pitch);
			
			Flags GetFlags() const {return _flags; }
			float GetGain() const;
			float GetPitch() const;
			
			void Update(float delta) override;
			
		private:
			void EmptyQueue();
			void StreamData();
			
			Flags _flags;
			Vector3 _previousPosition;
			
			Sound *_sound;
			uint32 _buffers[kRNOpenALBufferCount];
			uint32 _source;
			size_t _index;
			
			float _rolloffFactor;
			bool _awaitsStop;
			bool _isStreaming;
		};
	}
}

#endif /* defined(__rayne_openal__RASource__) */
